@@ -1,6 +1,6 @@
 # E2::Room.pm
 # Jose M. Weeks <jose@joseweeks.com>
-# 04 May 2003
+# 05 June 2003
 #
 # See bottom for pod documentation.
 
@@ -14,7 +14,8 @@ use Carp;
 use E2::Node;
 
 our @ISA = "E2::Node";
-our $VERSION = "0.30";
+our $VERSION = "0.31";
+our $DEBUG; *DEBUG = *E2::Interface::DEBUG;
 
 # Prototypes
 
@@ -45,6 +46,8 @@ sub new {
 sub clear {
 	my $self = shift	or croak "Usage: clear E2USERGROUP";
 
+	warn "E2::Room::clear\n"	if $DEBUG > 1;
+
 	$self->{description}	= undef;
 	$self->{can_enter}	= undef;
 
